/******************************************************************************/
/*                              CLAGLUE.H                                     */
/*                                                                            */
/*  CLAGLUE.H defines types and macros, and declares functions that are       */
/*  generally useful for writing modules to glue third-party code into        */
/*  Clarion applications.                                                     */
/*                                                                            */
/*  Sean Wilson, 16 July 1992                                                 */
/******************************************************************************/

#ifndef CLAGLUE_H
#define CLAGLUE_H

/*** The following macro defines the Clarion calling & naming conventions.  ***/
/*** This macro should be used when declaring C functions to interface with ***/
/*** Clarion in order to ensure future compatibility.                       ***/

#define CLA_CONV        name(prefix=>"", upper_case=>on)

/*** NOTE: C++ compiled functions which may be called from Clarion 3.0 must ***/
/*** be declared within a 'extern "Pascal" {...};' modifier.  The CLA_CONV  ***/
/*** pragma will not affect the name mangling employed by the C++ compiler. ***/

/*** Type equivalences.  The CLA prefix is used to avoid name clashes with  ***/
/*** third party libraries.                                                 ***/

typedef unsigned char   CLABYTE;
typedef short           CLASHORT;
typedef unsigned short  CLAUSHORT;
typedef long            CLALONG;
typedef unsigned long   CLAULONG;
typedef float           CLASREAL;
typedef double          CLAREAL;

/*** Clarion STRINGS are passed as two parameters, first an unsigned short  ***/
/*** is passed which contains the length of the data buffer, second the     ***/
/*** address of the data buffer.  If the RAW attribute is used in the       ***/
/*** Clarion procedure prototype then only the address of the data buffer   ***/
/*** is passed.  The CLASTRING structure is illustrative and cannot be used ***/
/*** to accept parameters from Clarion.                                     ***/

/*** typedef struct {CLAUSHORT ucLen; CLABYTE *pucString;} CLASTRING;       ***/

/*** Clarion PSTRINGS are passed as two parameters, first a CLABYTE is      ***/
/*** passed which contains the length of the string in the data buffer,     ***/
/*** second the address of the data buffer.  If the RAW attribute is used   ***/
/*** in the Clarion procedure prototype then only the address of the data   ***/
/*** buffer is passed.  The CLAPSTRING structure is illustrative and cannot ***/
/*** be used to accept parameters from Clarion.                             ***/

/*** typedef struct {CLABYTE ucLen; CLABYTE ucString[255];} CLAPSTRING;     ***/

/*** Clarion CSTRINGS are passed as two parameters, first a CLAUSHORT is    ***/
/*** passed which contains the length of the string in the data buffer,     ***/
/*** second the address of the data buffer.  If the RAW attribute is used   ***/
/*** in the Clarion procedure prototype then only the address of the data   ***/
/*** buffer is passed.  The CLACSTRING structure is illustrative and cannot ***/
/*** be used to accept parameters from Clarion.                             ***/

/*** typedef struct {CLAUSHORT ucLen; CLABYTE *pucString;} CLACSTRING;      ***/


/*** Clarion DATEs are passed as a CLALONG, the CLADATE union can be used   ***/
/*** to resolve the elements of the date from the CLALONG value.            ***/

typedef union {
    CLALONG         l;
    struct {
        CLABYTE     ucDay;
        CLABYTE     ucMonth;
        CLAUSHORT   usYear;
    }               s;
}   CLADATE;

/*** Clarion TIMEs are passed as a CLALONG, the CLADATE union can be used   ***/
/*** to resolve the elements of the time from the CLALONG value.            ***/

typedef union {
    CLALONG         l;
    struct {
        CLABYTE     ucHund;
        CLABYTE     ucSecond;
        CLABYTE     ucMinute;
        CLABYTE     ucHour;
    }               s;
}   CLATIME;


#define CLATRUE     1
#define CLAFALSE    0

#ifdef __cplusplus
extern "Pascal" {                       /* Use Clarion conventions            */
#else
#pragma save, CLA_CONV
#endif

/** MAPADDR *******************************************************************
 *
 *  Synopsis:
 *      INCLUDE('ClaGlue.Cla')
 *
 *      MapAddr(ULONG, STRING)
 *
 *  Description:
 *      MapAddr() is called from a Clarion program to map a group structure to
 *      an address.  This mapping is acheived only by copying data from the
 *      address expressed as a ULONG to the group passed as a STRING parameter.
 *      This procedure is intended to be used only to allow a Clarion program
 *      to map a group onto a pointer to a structure returned from a C function.
 */

void MapAddr(CLAULONG ulAddr, CLAUSHORT usLen, CLABYTE *cpData);


#ifdef __cplusplus
}                                       /* Restore C++ conventions            */
#else
#pragma restore                         /* Restore C conventions              */
#endif

#endif /* CLAGLUE_H */
